#include <windows.h>
#include <io.h>
#include <string.h>
#include <memory.h>
#include <fcntl.h>
#include <dinput.h>

#include "type.h"
#include "joydrv.h"
#include "win32def.h"

#define RETURN { JOY_CLOSE(); return FPSE_ERR; }

#define DIRELEASE(d) if (d != NULL) { d->lpVtbl->Release(d); d = NULL; }

#define DINPUT_BUFFERSIZE  16

DIPROPDWORD dipdw = {
// the header
        {
            sizeof(DIPROPDWORD),        // diph.dwSize
            sizeof(DIPROPHEADER),       // diph.dwHeaderSize
            0,                          // diph.dwObj
            DIPH_DEVICE,                // diph.dwHow
        },
// the data
        DINPUT_BUFFERSIZE,              // dwData
};

// DirectInput vars
static LPDIRECTINPUT       lpdi      = NULL;
static LPDIRECTINPUTDEVICE lpdiMouse = NULL;

static HANDLE  g_hevtMouse;
static char   *padbuf = NULL;

//-----------------------------------------------------------------

// This Function is called when emulator starts
int JOY_OPEN(UINT32 *par)
{
    FPSEWin32     *joy = (FPSEWin32 *)par;
    HRESULT        hr; 
//
    hr = DirectInputCreate(joy->HInstance,
			DIRECTINPUT_VERSION, &lpdi, NULL); 
    if (hr != DI_OK) RETURN
    hr = IDirectInput_CreateDevice(lpdi, &GUID_SysMouse, &lpdiMouse, NULL);
    if (FAILED(hr)) RETURN
    hr = IDirectInputDevice_SetDataFormat(lpdiMouse,&c_dfDIMouse);
    if (FAILED(hr)) RETURN
    hr = IDirectInputDevice_SetCooperativeLevel(lpdiMouse,joy->HWnd,
               DISCL_EXCLUSIVE | DISCL_FOREGROUND);
    if (FAILED(hr)) RETURN

    g_hevtMouse = CreateEvent(0, 0, 0, 0);
    if (g_hevtMouse == NULL) RETURN;
    hr = IDirectInputDevice_SetEventNotification(lpdiMouse,g_hevtMouse);
    if (FAILED(hr)) RETURN

    hr = IDirectInputDevice_SetProperty(lpdiMouse,DIPROP_BUFFERSIZE, &dipdw.diph);
    if (FAILED(hr)) RETURN

    hr = IDirectInputDevice_Acquire(lpdiMouse);
    if (FAILED(hr)) RETURN

    return FPSE_OK;
}

// This Function is called when emulator is closed
void JOY_CLOSE()
{
    DIRELEASE(lpdiMouse);
    DIRELEASE(lpdi);
}

// This function is called when the gamepad should vibrate:
// if (motor != 0) then vibrate(on) else vibrate(off)
void JOY_VIBRATE(int motor)
{
}

// Return status of buttons (see joydrv.h)
void JOY_POLL()
{
    DIMOUSESTATE    mouse;
    HRESULT         res;
    int             x;

    do
    {
        res = IDirectInputDevice_GetDeviceState(lpdiMouse, sizeof(mouse), &mouse);
        if (res == DIERR_INPUTLOST ||
            res == DIERR_NOTACQUIRED) IDirectInputDevice_Acquire(lpdiMouse);
    } while (res == DIERR_INPUTLOST);

    if (res != DI_OK)
    {
        padbuf[4] = 0xFF;
        padbuf[5] = 0;
        padbuf[6] = 0;
        return;
    }

    x = mouse.lX;
    if (x < -128) x = -128;
        else
    if (x > 127) x = 127;
    padbuf[5] = x;

    x = mouse.lY;
    if (x < -128) x = -128;
        else
    if (x > 127) x = 127;
    padbuf[6] = x;

    x = 0xFF;
    if (mouse.rgbButtons[0] & 0x80) x &= ~0x08;
    if (mouse.rgbButtons[1] & 0x80) x &= ~0x04;
    padbuf[4] = x;
}

int JOY_INIT(char *buf)
{
    padbuf = buf;
    buf[1] = 0x12;
    buf[2] = 0x5A;
    buf[3] = 0xFF;
    buf[4] = 0xFF;
    buf[5] = 0;
    buf[6] = 0;
    return 7;
}

#define VERSION     0x0100  // Version must be in BCD format

static FPSEWin32About info = {
#ifdef __PSX_PAD_0__
    FPSE_JOY0,
#else
    FPSE_JOY1,
#endif
    VERSION & 0xFF,
    VERSION >> 8,
    0,
    "LDChen",
    "DMouse",
    "DirectInput Mouse Driver\x00d\x00a\x00d\x00a"
    "This plugin emulates the input device SCPH-1030.\x00d\x00a"
    "Requires DirectX 3.0"
};

int JOY_CONFIG(UINT32 *par)
{
    FPSEWin32 *info = (FPSEWin32 *)par;

    MessageBox(info->HWnd,"This Plugin doesn't need to be configured",
               "DMouse config",MB_OK);

    return FPSE_OK;
}

void JOY_ABOUT(UINT32 *par)
{
    memcpy( par, &info, sizeof(FPSEWin32About) );
}
